#ifndef BUILDING_HPP_INCLUDED
#define BUILDING_HPP_INCLUDED

#include <boost/shared_ptr.hpp>

#include <string>

enum RESOURCE_TYPE { RESOURCE_GRAIN, RESOURCE_VENISON, RESOURCE_FISH, NUM_RESOURCES };

std::string resource_name(int nresource);

class building;
class player;

typedef boost::shared_ptr<building> building_ptr;
typedef boost::shared_ptr<const building> const_building_ptr;

class building
{
public:
	static int num_buildings();
	static const_building_ptr get_building(int id);
	static int get_building_id(const std::string& name);

	building(const std::string& name, int grain, int venison, int fish, int wild, int culture, int points, bool unique, const std::string& help);
	virtual ~building();

	virtual int ai_evaluate(const player& p) const = 0;

	void build(player& p) const;

	void set_id(int id) { id_ = id; }
	int id() const { return id_; }
	const std::string& name() const { return name_; }
	const std::string& help() const { return help_; }

	const int* cost() const { return cost_; }
	bool unique() const { return unique_; }
	int ninstances() const { return ninstances_; }
	int culture() const { return culture_; }
	int victory_points() const { return points_; }

private:
	virtual void on_build(player& p) const {}

	int id_;
	std::string name_;
	int cost_[NUM_RESOURCES+1];
	int culture_;
	int points_;
	bool unique_;
	
	mutable int ninstances_;
	std::string help_;
};

#endif
