#include <ctype.h>

#include "building_widget.hpp"
#include "font.hpp"
#include "foreach.hpp"
#include "formatter.hpp"
#include "raster.hpp"
#include "string_utils.hpp"
#include "texture.hpp"

building_widget::building_widget(player_ptr p, const_building_ptr b, boost::function<void()> on_click)
  : player_(p), building_(b), on_click_(on_click),
  culture_(font::render_text(formatter() << b->culture(), graphics::color_green(), 10)),
  victory_points_(font::render_text(formatter() << b->victory_points(), graphics::color_white(), 12))
{
	std::vector<std::string> title_str = util::split(b->name(), '_');
	std::string title;
	foreach(std::string s, title_str) {
		if(s.empty()) {
			continue;
		}

		s[0] = toupper(s[0]);
		title_.push_back(font::render_text(s, graphics::color_yellow(), 12));
		title_gray_.push_back(font::render_text(s, graphics::color_gray(), 12));
		title += s + " ";
	}

	set_tooltip(formatter() << title << ": " << b->help());

	set_dim(75, 46);
}

void building_widget::handle_draw() const
{
	const SDL_Rect rect = {x(),y(),width(),height()};
	graphics::draw_rect(rect, graphics::color_black());

	const bool owns = player_->has_building(building_->id());
	if(owns) {
		graphics::draw_rect(rect, player::get_color(player_->number()), 64);
	}
	graphics::draw_hollow_rect(rect, graphics::color_white());

	int ypos = rect.y;
	const std::vector<graphics::texture>* title_vec = player_->can_afford_building(building_->id()) ? &title_ : &title_gray_;
	foreach(const graphics::texture& t, *title_vec) {
		graphics::blit_texture(t, rect.x + rect.w/2 - t.width()/2, ypos);
		ypos += 12;
	}

	graphics::blit_texture(culture_, rect.x, rect.y);

	graphics::blit_texture(victory_points_, rect.x + rect.w - victory_points_.width() - 4, rect.y + 5);

	int resource_x = x() + 5;
	for(int n = 0; n != 4; ++n) {
		for(int m = 0; m != building_->cost()[n]; ++m) {
			graphics::blit_texture(n == 3 ? player::wildcard_resource_icon() : player::development_icon(static_cast<DEVELOPMENT_TYPE>(n)), resource_x, y() + 24, 16, 16);
			resource_x += 10;
		}
	}

	if(building_->unique()) {
		static const graphics::texture unique_texture = font::render_text("unique", graphics::color_green(), 12);
		graphics::blit_texture(unique_texture, rect.x + rect.w - unique_texture.width(), rect.y + rect.h - unique_texture.height());
	}
}

bool building_widget::handle_event(const SDL_Event& event, bool claimed)
{
	if(event.type == SDL_MOUSEBUTTONDOWN) {
		if(event.button.x >= x() && event.button.x < x() + width() &&
		   event.button.y >= y() && event.button.y < y() + height()) {
			on_click_();
			return true;
		}
	}
	return false;
}
