
/*
   Copyright (C) 2007 by David White <dave.net>
   Part of the Silver Tree Project

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 or later.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/
#include "button.hpp"
#include "raster.hpp"
#include "surface_cache.hpp"
#include "framed_gui_element.hpp"

namespace gui {

namespace {

int vpadding = 4;
int hpadding = 10;

}

button::button(widget_ptr label, boost::function<void ()> onclick)
  : label_(label), onclick_(onclick),
	normal_button_image_set_(framed_gui_element::get("regular_button")),
	depressed_button_image_set_(framed_gui_element::get("regular_button_pressed")),
	focus_button_image_set_(framed_gui_element::get("regular_button_focus")),
	current_button_image_set_(normal_button_image_set_),
	enabled_(true)
	
{
	set_dim(label_->width()+hpadding*2,label_->height()+vpadding*2);
}

bool button::in_button(int xloc, int yloc) const
{
	return xloc > x() && xloc < x() + width() &&
	       yloc > y() && yloc < y() + height();
}

void button::handle_draw() const
{
	if(!enabled()) {
		glColor4f(0.5, 0.5, 0.5, 1.0);
	}

	label_->set_loc(x()+hpadding,y()+vpadding);
	current_button_image_set_->blit(x(),y(),width(),height());
	label_->draw();

	if(!enabled()) {
		glColor4f(1.0, 1.0, 1.0, 1.0);
	}
}

bool button::handle_event(const SDL_Event& event, bool claimed)
{
    if(claimed || !enabled()) {
		current_button_image_set_ = normal_button_image_set_;
    }

	if(!enabled()) {
		return false;
	}

	if(event.type == SDL_MOUSEMOTION) {
		const SDL_MouseMotionEvent& e = event.motion;
		if(current_button_image_set_ == depressed_button_image_set_) {
			//pass
		} else if(in_button(e.x,e.y)) {
			current_button_image_set_ = focus_button_image_set_;
		} else {
			current_button_image_set_ = normal_button_image_set_;
		}
	} else if(event.type == SDL_MOUSEBUTTONDOWN) {
		const SDL_MouseButtonEvent& e = event.button;
		if(in_button(e.x,e.y)) {
			current_button_image_set_ = depressed_button_image_set_;
		}
	} else if(event.type == SDL_MOUSEBUTTONUP) {
		const SDL_MouseButtonEvent& e = event.button;
		if(current_button_image_set_ == depressed_button_image_set_) {
			if(in_button(e.x,e.y)) {
				current_button_image_set_ = focus_button_image_set_;
				onclick_();
				claimed = true;
			} else {
				current_button_image_set_ = normal_button_image_set_;
			}
		}
	}
	return claimed;
}

void button::set_enabled_calculator(boost::function<bool()> fn)
{
	enabledfn_.reset(new boost::function<bool()>(fn));
}

bool button::enabled() const
{
	if(enabledfn_.get() != NULL) {
		return (*enabledfn_)();
	}

	return enabled_;
}

}
