#include <GL/gl.h>

#include "deity_widget.hpp"
#include "font.hpp"
#include "formatter.hpp"
#include "game.hpp"
#include "player.hpp"
#include "raster.hpp"

deity_widget::deity_widget(const game& g, const_deity_ptr d, boost::function<void()> on_click)
  : game_(g), deity_(d), on_click_(on_click),
    title_(font::render_text(d->name(), graphics::color_yellow(), 24))
{
	set_tooltip(formatter() << deity_->name() << "\n" << d->help());
	set_dim(140, 160);
}

int deity_widget::priest_xpos(int npriest) const
{
	return x() + 45 + (npriest <= 1 ? 0 : npriest-1)*35;
}

int deity_widget::priest_ypos(int npriest) const
{
	return y() + (npriest == 0 ? 45 : 85);
}

void deity_widget::handle_draw() const
{
	int mousex, mousey;
	SDL_GetMouseState(&mousex, &mousey);

	const bool mouseover = mousex >= x() && mousex < x() + width() && mousey >= y() && mousey < y() + height();

	const SDL_Rect rect = {x(),y(),width(),height()};
	graphics::draw_rect(rect, graphics::color_black());
	if(mouseover) {
		graphics::draw_rect(rect, graphics::color_yellow(), 64);
	}

	graphics::draw_hollow_rect(rect, graphics::color_white());
	graphics::blit_texture(title_, rect.x + rect.w/2 - title_.width()/2, rect.y);

	const char alpha_str[] = {0xE2, 0x8D, 0xB6, 0x00};
	static const graphics::texture alpha = font::render_text("A", graphics::color_yellow(), 22);
	static const graphics::texture beta = font::render_text("B", graphics::color_yellow(), 22);
	graphics::blit_texture(alpha, rect.x + 5, rect.y + 30);
	graphics::blit_texture(beta, rect.x + 5, rect.y + 70);

	const SDL_Rect priest_rect = {rect.x + 30, rect.y + 30, 30, 30};
	graphics::draw_hollow_rect(priest_rect, graphics::color_white());

	if(!deity_->priests().empty() && !game_.players()[deity_->priests().front()]->priest_in_transit_to(deity_->number())) {
		graphics::draw_circle(priest_rect.x + priest_rect.w/2, priest_rect.y + priest_rect.h/2, 12, player::get_color(deity_->priests().front()));
	}

	for(int n = 0; n != 3; ++n) {
		const SDL_Rect priest_rect = {rect.x + 30 + n*35, rect.y + 70, 30, 30};
		graphics::draw_hollow_rect(priest_rect, graphics::color_white());

		if(n+1 < deity_->priests().size() && !game_.players()[deity_->priests()[n+1]]->priest_in_transit_to(deity_->number())) {
			graphics::draw_circle(priest_rect.x + priest_rect.w/2, priest_rect.y + priest_rect.h/2, 12, player::get_color(deity_->priests()[n+1]));
		}
	}

	if(deity_->number() < 6) {
		const DEVELOPMENT_TYPE devel_type = static_cast<DEVELOPMENT_TYPE>(deity_->number());
		graphics::blit_texture(player::development_icon(devel_type), x() + 20, y() + 110);
	} else if(deity_->number() == 6) {
		if(deity_->plague_unleashed()) {
			graphics::blit_texture(font::render_text("PLAGUE!!", graphics::color_red(), 18), x() + 20, y() + 110);
		} else {
			//apollo -- victory points
			graphics::blit_texture(player::victory_points_icon(), x() + 20, y() + 110);
		}
	} else if(deity_->number() == 7) {
		//can build buidings.
		graphics::blit_texture(font::render_text("Building", graphics::color_yellow(), 18), x() + 20, y() + 110);
	} else if(deity_->number() == 8) {
		graphics::blit_texture(font::render_text("Choose", graphics::color_yellow(), 18), x() + 20, y() + 110);
	} else {
		for(int n = 0; n != 3; ++n) {
			const DEVELOPMENT_TYPE devel_type = static_cast<DEVELOPMENT_TYPE>(n);
			graphics::blit_texture(player::development_icon(devel_type), x() + 20 + n*35, y() + 110);
		}
	}

	if(deity::active().empty() == false && std::count(deity::active().begin(), deity::active().end(), deity_->number()) == 0) {
		graphics::draw_rect(rect, graphics::color_black(), 196);
	}

}

bool deity_widget::handle_event(const SDL_Event& event, bool claimed)
{
	if(event.type == SDL_MOUSEBUTTONDOWN) {
		if(event.button.x >= x() && event.button.x < x() + width() &&
		   event.button.y >= y() && event.button.y < y() + height()) {
			on_click_();
			return true;
		}
	}
	return false;
}
