#ifndef GAME_DIALOG_HPP_INCLUDED
#define GAME_DIALOG_HPP_INCLUDED

#include <deque>
#include <vector>

#include <boost/array.hpp>
#include <boost/function.hpp>

#include "building.hpp"
#include "dialog.hpp"
#include "player.hpp"
#include "widget.hpp"

class deity_widget;
class game;
class priests_widget;

class game_dialog : public gui::dialog
{
public:
	static game_dialog& get();
	explicit game_dialog(game& g);

	gui::dialog* set_message(player* p, const std::string& msg);
	void clear_message();

	void make_selection(int nselect);
	int modal_selection();
	void modal_info_message(const std::string& msg);

private:
	void init();
	void do_processing();
	void handle_draw() const;
	void select_deity(int n);
	void select_building(player_ptr p, const_building_ptr b);
	void select_building_payment(player_ptr p, const_building_ptr b, const std::vector<boost::array<int, 4> >& v, int n);

	void deity_selected(player_ptr p, int n);
	void building_selected(player_ptr p, const_building_ptr b);

	void setup_active_player();
	void setup_secondary_player(int ndeity);

	void declare_war(player_ptr p, int nenemy);
	void get_tribute(player_ptr p, player_ptr q, int resource, int ntributes_left);
	void show_tribute_dialog(player_ptr p, player_ptr q, int tribute);
	void select_resource(player_ptr p);
	void select_development(player_ptr p);
	void exchange_resources(player_ptr p);
	void exchange_resource_for_vp(player_ptr p);
	void exchange_resource_for_development(player_ptr p);
	void choose_resource(player_ptr p, bool choosing_resource, const char* msg=NULL);
	int modal_resource_selection(player_ptr p, const std::string& msg, int num=1, int noptions=3);
	void resource_selected(int n);
	void next_active_player();

	void player_chooses_deity(int n);
	void player_secondary_done(int n);

	void maintenance_phase();

	void select_viewed_player(int nplayer);

	boost::function<void()> network_function(boost::function<void()>);
	boost::function<void(int)> network_function_grid(boost::function<void(int)>);

	game& game_;
	int viewed_player_;

	int first_active_player_;
	int active_player_;
	int secondary_player_;

	gui::widget_ptr message_dialog_;
	gui::widget_ptr players_grid_;
	gui::widget_ptr network_dialog_;

	struct widget_modify_operation {
		gui::widget_ptr widget;
		bool add;
		int x, y;
	};

	std::vector<widget_modify_operation> widget_modify_queue_;

	std::vector<gui::widget_ptr> widget_trash_;

	std::vector<const deity_widget*> deity_widgets_;
	priests_widget* priests_widget_;

	struct scheduled_command {
		boost::function<void()> cmd;
		int cycles;
	};

	std::deque<scheduled_command> scheduled_commands_;

	void schedule_command(boost::function<void()> cmd, int cycles);

	//generic functionality for selecting options from a list.
	int dialog_selection_;

	player* input_player_;
	std::vector<boost::function<void()> > network_functions_;
};

#endif
