#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#include <boost/bind.hpp>

#include <algorithm>
#include <iostream>

#include "asserts.hpp"
#include "button.hpp"
#include "client_network.hpp"
#include "dialog.hpp"
#include "filesystem.hpp"
#include "flags.hpp"
#include "foreach.hpp"
#include "formatter.hpp"
#include "font.hpp"
#include "framed_gui_element.hpp"
#include "frame_rate_handler.hpp"
#include "game.hpp"
#include "game_dialog.hpp"
#include "label.hpp"
#include "random.hpp"
#include "raster.hpp"
#include "texture.hpp"
#include "widget.hpp"
#include "wml_node.hpp"
#include "wml_parser.hpp"
#include "wml_utils.hpp"

namespace {
bool local_game = true;
int global_num_players = 1;
int global_player_slot = 0;
void select_game_type(gui::dialog* d, bool value)
{
	d->close();
	local_game = value;
}
using namespace gui;

class lobby_dialog : public gui::dialog {
public:

	lobby_dialog() : dialog(0, 0, 1024, 768), ncycles_(0) {
		players_label_ = new label(formatter() << "Players: (updating)", graphics::color_white());
		add_widget(widget_ptr(players_label_), 500, 320);
		add_widget(widget_ptr(new button(widget_ptr(new label("Start Game", graphics::color_white())), boost::bind(&lobby_dialog::start_game, this))));
	}

	void do_processing() {
		wml::const_node_ptr node = network::receive();
		if(node) {
			std::cerr << "GOT MESSAGE...\n";
			if(node->name() == "game") {
				std::cerr << "SET TEXT\n";
				players_label_->set_text(formatter() << "Players: " << wml::get_int(node, "players"));
			} else if(node->name() == "start_game") {
				global_num_players = wml::get_int(node, "players");
				global_player_slot = wml::get_int(node, "position");
				close();
			}
		}

		static const std::string UpdateGameStr = "[update_game]\n[/update_game]\n";
		if((ncycles_%100) == 0) {
			network::send(UpdateGameStr);
		}

		++ncycles_;
	}

	void start_game() {
		network::send("[start_game]\n[/start_game]\n");
	}

private:
	label* players_label_;
	int ncycles_;
};

}

void play_game()
{
	rng::set_seed(1);

	try {
		using namespace gui;
		dialog d(0, 0, 1024, 768);
		d.add_widget(widget_ptr(new button(widget_ptr(new label("Local Game", graphics::color_white())), boost::bind(select_game_type, &d, true))), 500, 320);
		d.add_widget(widget_ptr(new button(widget_ptr(new label("Network Game", graphics::color_white())), boost::bind(select_game_type, &d, false))), 500, 380);
		d.show_modal();

		if(local_game) {
			game g;
			game_dialog dialog(g);
			dialog.show_modal();
		} else {
			network::manager network_manager;
			network::connect("wesnoth.org", "17000");
			lobby_dialog d;
			d.show_modal();

			game g(global_player_slot, global_num_players);
			game_dialog dialog(g);
			dialog.show_modal();
		}
	} catch(game_over_exception&) {
		std::cerr << "GAME OVER!\n";
	}
}

int main(int argc, char** argv)
{
	srand(time(NULL));

	flags::init(argv);

	const int screen_width = flags::get_int("width", 1024);
	const int screen_height = flags::get_int("height", 768);
	const bool fullscreen = false;
	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0) {
		std::cerr << "could not init SDL\n";
		return -1;
	}

	if(SDL_SetVideoMode(screen_width,screen_height,0,SDL_OPENGL|(fullscreen ? SDL_FULLSCREEN : 0)) == NULL) {
		std::cerr << "could not set video mode\n";
		return -1;
	}

	{
	graphics::texture::manager texture_manager;
	font::manager font_manager;

	framed_gui_element::init(wml::parse_wml_from_file("gui.cfg"));

	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnableClientState(GL_VERTEX_ARRAY);

	play_game();
	}

	SDL_Quit();
}
