#include "formatter.hpp"
#include "font.hpp"
#include "raster.hpp"
#include "slider_widget.hpp"

slider_widget::slider_widget(player_ptr p, DEVELOPMENT_TYPE type)
  : player_(p), type_(type), icon_(player::development_icon(type))
{
	set_dim(200, 30);
}

void slider_widget::handle_draw() const
{
	graphics::blit_texture(icon_, x(), y());
	int xpos = x() + 60;
	const int square_dim = 30;
	int nproductivity = 1;
	for(int n = 0; n <= player_->max_development(type_); ++n) {
		const SDL_Rect rect = {xpos, y(), square_dim, height()};
		graphics::draw_hollow_rect(rect, graphics::color_white());

		if(type_ <= 2) {
			const int productivity = player::resource_productivity(n);
			for(int row = 0; row != 2; ++row) {
				const int ydraw = y() + ((productivity > 2) ? row*16 : 8);
				for(int col = 0; col != 2; ++col) {
					const int xdraw = xpos + ((productivity == 1 || row == 0 && productivity&1) ? 8 : col*16);
					graphics::blit_texture(icon_, xdraw, ydraw, 16, 16);
				}
			}
		} else {
			graphics::texture t = font::render_text(formatter() << (n+1), graphics::color_white(), 16);
			graphics::blit_texture(t, rect.x + rect.w/2 - t.width()/2, rect.y + rect.h/2 - t.height()/2);
		}

		if(n > player_->max_building_development(type_)) {
			graphics::draw_rect(rect, graphics::color_black(), 196);
		}

		xpos += square_dim;
	}

	const int circle_pos = x() + 60 + square_dim/2 + (player_->development_draw(type_)*square_dim)/100;
	graphics::draw_circle(circle_pos, y() + height()/2, 10, player::get_color(player_->number()));

	const int glory_owner = player::glory_owner(type_);
	graphics::texture glory_tex = font::render_text("+2VP", glory_owner == -1 ? graphics::color_gray() : player::get_color(glory_owner), 16);
	graphics::blit_texture(glory_tex, xpos+5, y());
}
