
/*
   Copyright (C) 2007 by David White <dave.net>
   Part of the Silver Tree Project

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 or later.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/
#include "SDL.h"

#include "font.hpp"
#include "foreach.hpp"
#include "raster.hpp"
#include "tooltip.hpp"

namespace gui {

namespace {
boost::shared_ptr<std::vector<std::string> > cur_tooltip;

std::vector<graphics::texture>& text() {
	static std::vector<graphics::texture> t;
	return t;
}

int tooltip_width, tooltip_height;
}

void set_tooltip(const boost::shared_ptr<std::vector<std::string> >& tip)
{
	cur_tooltip = tip;
	text().clear();
	tooltip_width = tooltip_height = 0;
	for(int n = 0; n != cur_tooltip->size(); ++n) {
		text().push_back(font::render_text((*cur_tooltip)[n], graphics::color_yellow(), 18));
		if(text().back().width() > tooltip_width) {
			tooltip_width = text().back().width();
		}

		tooltip_height += text().back().height();
	}
}

void remove_tooltip(const boost::shared_ptr<std::vector<std::string> >& tip)
{
	if(tip == cur_tooltip) {
		cur_tooltip.reset();
		text().clear();
		tooltip_width = tooltip_height = 0;
	}
}

void draw_tooltip()
{
	if(!cur_tooltip) {
		return;
	}

	int mousex, mousey;
	SDL_GetMouseState(&mousex,&mousey);

	const int pad = 10;
	const int width = tooltip_width + pad*2;
	const int height = tooltip_height + pad*2;
	int x = mousex - width/2;
	int y = mousey - height;
	if(x < 0) {
		x = 0;
	}

	if(x > graphics::screen_width()-width) {
		x = graphics::screen_width()-width;
	}

	if(y < 0) {
		y = 0;
	}

	if(y > graphics::screen_height()-height) {
		y = graphics::screen_height()-height;
	}

	SDL_Rect rect = {x,y,width,height};
	graphics::draw_rect(rect, graphics::color_black(), 160);

	int ypos = y+pad;
	foreach(const graphics::texture& t, text()) {
		graphics::blit_texture(t,x+pad,ypos);
		ypos += t.height();
	}
}

}
